@@ -6,9 +6,9 @@ MSG="doc build for commit $GITHASH"
 cd _build
 
 # clone the repo if needed
-if test -d altair-viz.github.io; 
+if test -d altair-viz.github.io;
 then echo "using existing cloned altair directory";
-else git clone git@github.com:altair-viz/altair-viz.github.io.git;
+else git clone https://github.com/altair-viz/altair-viz.github.io.git;
 fi
 
 # sync the website
