@@ -2,8 +2,7 @@
 schemapi: tools for generating Python APIs from JSON schemas
 """
 from .schemapi import SchemaBase, Undefined
-from .decorator import schemaclass
 from .utils import SchemaInfo
 
 
-__all__ = ("SchemaBase", "Undefined", "schemaclass", "SchemaInfo")
+__all__ = ("SchemaBase", "Undefined", "SchemaInfo")
